%%%% Rice University Dataset %%%%
load Rice31.mat
covlen=3; % Number of covariates %
%%%% Covariate Values %%%%
dd1=local_info(:,5);
dd2=local_info(:,2);
dd3=local_info(:,6);
ddind=find(dd3 > 2003 & dd3 < 2011); % Restrict only to Facebook data 2003-2011
%%%% Storing the covariate values for the relevant period %%%%
datamat1=dd1(ddind);
datamat2=dd2(ddind);
datamat3=dd3(ddind);
%%%% Excluding the Missing Values %%%%
zero1=find(datamat1==0);
zero2=find(datamat2==0);
zero3=find(datamat3==0);
zeroc=vertcat(zero1,zero2,zero3);
datamat1(zeroc)=[];
datamat2(zeroc)=[];
datamat3(zeroc)=[];
A=A(ddind,ddind);
A(zeroc,:)=[];
A(:,zeroc)=[];
S=sum(A,2);% Degree of each node in the network

%%%% Removing Nodes with degrees <= 1
ind=find(S<=1);
A(ind,:)=[];
A(:,ind)=[];
n = size(A,1); % number of nodes
datamat1(ind,:)=[];
datamat2(ind,:)=[];
datamat3(ind,:)=[];
S(ind)=[];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lambda=mean(S); % Average degree of the network %


%%%%%%%%% Key parameters of SBM with covariate %%%%%%%%%
K = 20;
nsamp_grp=50; %%% Number of individuals sampled per cluster %%%
compErr = @(c,e) compMuI(compCM(c,e,K));    % use mutual info as a measure of error/sim.
niter=30; % number of iterations %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Initialization via spectral clustering %%%
init_opts = struct('verbose',false);
[e init_dT] = initLabel5b(A, K, 'scp', init_opts);
cinit=e; % Initial labeling

%%%%%% Opening Parallel Connection in Matlab %%%%%%%
ncore=30;
nsubsamp=30;
matlabpool ('open',ncore);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Data Subsampling %%%
mv_samp=cell(nsubsamp,1); % Selected subsamples
nsamp_ttl=zeros(nsubsamp,1); % Total number of subsamples
samplesize_vec=cell(nsubsamp,1); % Ratio of number of individuals sampled from each class
for ncr=1:nsubsamp
    m_samp=cell(K,1);
    samplesize.vector=zeros(K,1);
    for i=1:K
        if(length(find(cinit==i))<=nsamp_grp)
            m_samp{i}=find(cinit==i);
        else
            m_samp{i}=randsample(find(cinit==i),nsamp_grp);
        end
        samplesize.vector(i)=sum(cinit==i)/min(nsamp_grp,sum(cinit==i));
    end
    mv_samp{ncr}=cell2mat(m_samp);
    samplesize_vec{ncr}=samplesize.vector;
    %mv_samp=sort(mv_samp_unsort);
    nsamp_ttl(ncr)=K*nsamp_grp;
end
%%%% Storing estimates for different subsamples %%%%
pi_parhat=cell(nsubsamp,1);
thetalink_parhat=cell(nsubsamp,1);
cinit_par=cell(nsubsamp,1);
beta_incompcc_par=cell(nsubsamp,1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pi_init=zeros(K,1); %%% initial class probability estimates %%%
for i=1:(K-1)
    pi_init(i)=length(cinit(cinit==i))/n;
end
pi_init(K)=1-sum(pi_init(1:(K-1)));

%%%% Intialization of edge probability matrix of the latent part %%%%
thetalink_cc=zeros(K,K);
for u=1:K
    for v=1:K
        if(v>=u)
            thetalink_cc(u,v)=log(community_sum(A,u,v,cinit)/(ncomm(u,cinit)*ncomm(v,cinit)-community_sum(A,u,v,cinit)));
        else
            thetalink_cc(u,v)=thetalink_cc(v,u);
        end
    end
end

thetalink=thetalink_cc;


%%% Covariate values in the form of a array %%%
X=zeros(n,n,covlen);
X1=pdist(datamat1,'hamming');
X2=pdist(datamat2,'hamming');
X3=pdist(datamat3,'hamming');
X(:,:,1)=squareform(X1);
X(:,:,2)=squareform(X2);
X(:,:,3)=squareform(X3);


%%% Initialization of parameter beta for the covariates %%%
Avec=reshape(A,n*n,1);
idx=reshape(eye(n),n*n,1);
Avec(idx==1)=[];
Xvec=reshape(X,n*n,covlen);
Xtrnsf=bsxfun(@minus,Xvec,Xvec(:,1));
Xtrnsf(:,1)=[];
Xvec(idx==1,:)=[];
Xtrn=bsxfun(@minus,Xvec,Xvec(:,1));
Xtrn(:,1)=[];
%%%% Simple logistic regression omitting the latent part %%%%
[beta_incompcc,dev,]=glmfit(Xtrn,Avec,'binomial', 'link','logit','constant','on');
beta_incompcc(1)=[];
beta_cc=[beta_incompcc;-sum(beta_incompcc)];% constraint- sum of beta's is 0
Xmat=reshape(Xtrnsf,n,n,(covlen-1)); % X(i,j) is a vector of covariates of order 2x1

%%% Assigning the initial values to each of the subsamples in each core
for ncr=1:nsubsamp
    thetalink_parhat{ncr}=thetalink;
    pi_parhat{ncr}=pi_init;
    cinit_par{ncr}=cinit;
    beta_incompcc_par{ncr}=beta_incompcc;
end

nz_vec=cell(nsubsamp,1); % Each cell contains the number of zero connections in each subsample
r=7; % global case-control rate %
n_sampnode=lambda*r; % Size of case-control sample
ns_ttl=K*nsamp_grp; % Total number of individuals sampled from all the groups
nzero_pop=zeros(ns_ttl,1); % Total number of zero connections for the selected nodes
zerocon_samp=zeros(n_sampnode,ns_ttl); % sampled zero connections in case-control sampling
zero_samp=cell(nsubsamp,1); % sampled zero connections over each subsample

%%%% SRS sampling over connections for each node %%%%
for ncr=1:nsubsamp
    mvsamp_vec=mv_samp{ncr};
    for i=1:nsamp_ttl
        zero_pop=find(A(mvsamp_vec(i),:)==0);
        nzero_pop(i)=length(zero_pop);
        zerocon_samp(:,i)=randsample(zero_pop,n_sampnode);
    end
    zero_samp{ncr}=zerocon_samp;
    nz_vec{ncr}=nzero_pop;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Monte-Carlo EM type algorithm implemented in parallel %%%
%tic;
est_theta=cell(nsubsamp,1); % Phat in each core %
est_pri=cell(nsubsamp,1); % Pihat in each core %
est_c=cell(nsubsamp,1); % NMI in each core %
est_beta=cell(nsubsamp,1); % betahat in each core %

%%% Iteration run in parallel over different cores %%%
for t=1:niter
    if t<11
        ngibbs=100; %number of gibbs samples %
    elseif ((t>=11) && (t<16))
        ngibbs=300; % number of gibbs samples %
    elseif ((t>=16) && (t<21))
        ngibbs=500; % number of gibbs samples %
    elseif ((t>=21) && (t<26))
        ngibbs=700; % number of gibbs samples %
    else
        ngibbs=1000; % number of gibbs samples %
    end
    parfor idx=1:ncore
        [est_c{idx},est_theta{idx},est_pri{idx},est_beta{idx}]=parEM_cov(ngibbs,beta_incompcc_par{idx},thetalink_parhat{idx},Xmat,cinit_par{idx},pi_parhat{idx},A,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
    end
    %parfor idx=11:20
        %[est_c{idx},est_theta{idx},est_pri{idx},est_beta{idx}]=parEM_cov(ngibbs,beta_incompcc_par{idx},thetalink_parhat{idx},Xmat,cinit_par{idx},pi_parhat{idx},A,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
    %end
     %parfor idx=21:nsubsamp
        %[est_c{idx},est_theta{idx},est_pri{idx},est_beta{idx}]=parEM_cov(ngibbs,beta_incompcc_par{idx},thetalink_parhat{idx},Xmat,cinit_par{idx},pi_parhat{idx},A,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
     %end
     
    %%%% Communication Step %%%%
    for idx=1:nsubsamp
        est_theta{idx}=est_theta{mod(idx,nsubsamp)+1};
        est_pri{idx}=est_pri{mod(idx,nsubsamp)+1};
        est_beta{idx}=est_beta{mod(idx,nsubsamp)+1};
        thetalink_parhat{idx}=est_theta{idx};
        pi_parhat{idx}=est_pri{idx};
        beta_incompcc_par{idx}=est_beta{idx};
        cinit_par{idx}=est_c{idx};
    end
end
%%%%%% END of iteration %%%%%%

%wtime = toc;
%fprintf ( 1, '  Main computation:\n' );
%fprintf ( 1, '    Wall clock time = %f seconds.\n', wtime );
matlabpool close;
thetafinal=zeros(K,K);
prifinal=zeros(K,1);
betafinal=zeros((covlen-1),1);

%%% Combining estimates from different cores %%%
for j=1:nsubsamp
    thetafinal=thetafinal+est_theta{j};
    prifinal=prifinal+est_pri{j};
    betafinal=betafinal+est_beta{j};
end
thetafinal=thetafinal/nsubsamp;
prifinal=prifinal/nsubsamp;
betafinal=betafinal/nsubsamp;
betahat_final=[betafinal;-sum(betafinal)];
Fmat=cell2mat(est_c);
Fmat=reshape(Fmat,[n nsubsamp]);
cfinal=mode(Fmat,2); % Majority voting for aggregating labels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% you must include an exit command in your Matlab .m file to close
% Matlab when your program is complete.
%exit

%%%%%%%%%%%%%%%%%%%%%% END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%