%%%% Key parameters of the Blockmodel without Covariate %%%%
n = 1000; % number of nodes
K = 3;    % number of communities
lambdaseq = [4,8,14];    % "O"ut-"I"n-"R"atio 
oir =0.04;    % average node degree
nsamp_grp=50; %%% Number of individuals sampled per cluster %%%
compErr = @(c,e) compMuI(compCM(c,e,K));    % use mutual info as a measure of error/sim.
niter=30; % number of iterations for MCMC %
repl_num=30; % number of replications %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%Arrays storing NMI's and estimation errors for our method and pseudo-likelihood%%%
cpl_nmi=zeros(length(lambdaseq),1);
cc_nmi=zeros(length(lambdaseq),1);
prierr_PL=zeros(length(lambdaseq),1);
Perr_PL=zeros(length(lambdaseq),1);
prierr=zeros(length(lambdaseq),1);
Perr=zeros(length(lambdaseq),1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Applying our Algorithm and Pseudo-likelihood for varying average
%%%% degree %%%%

for lambda_ind=1:length(lambdaseq)
    lambda=lambdaseq(lambda_ind); %% Average Degree %%
    %tic, fprintf('%-40s','Single replication ...')
    cpl_nmi(lambda_ind)=0;
    cc_nmi(lambda_ind)=0;
    prierr_PL(lambda_ind)=0;
    Perr_PL(lambda_ind)=0;
    prierr(lambda_ind)=0;
    Perr(lambda_ind)=0;
    for repl=1:repl_num
        inWei = [1 1 1];   % relative wieght of in-class probabilities
        [lowVal, lowProb] = deal(0.2,0); % An example of original block model
        mo = dcBlkMod2(n,K,lambda, lowVal, lowProb); % create a base model
        mo = mo.genP(oir, inWei);  % generate the edge probability matrix
        mo = mo.genData;% generate data (Adj. matrix "As" and the labels "c")
        
        
        % options for the init method and cpl
        init_opts = struct('verbose',false);
        T = 20;
        cpl_opts = struct('verbose',false,'delta_max',0.1, ...
            'itr_num',T,'em_max',80,'track_err',false);
        %%% Initialization via spectral clustering %%%
        [e, init_dT] = initLabel5b(mo.As, mo.K, 'scp', init_opts);
        cinit=e; % Initial labeling
        
        %%%%%% Opening Parallel Connection in Matlab %%%%%%%
        ncore=30;
        nsubsamp=30;
        matlabpool ('open',ncore);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%% Data Subsampling %%%
        mv_samp=cell(nsubsamp,1); % Selected subsamples
        nsamp_ttl=zeros(nsubsamp,1); % Total number of subsamples
        samplesize_vec=cell(nsubsamp,1); % Ratio of number of individuals sampled from each class
        for ncr=1:nsubsamp
            m_samp=cell(K,1);
            samplesize.vector=zeros(K,1);
            for i=1:K
                if(length(find(cinit==i))<=nsamp_grp)
                    m_samp{i}=find(cinit==i);
                else
                    m_samp{i}=randsample(find(cinit==i),nsamp_grp);
                end
                samplesize.vector(i)=sum(cinit==i)/min(nsamp_grp,sum(cinit==i));
            end
            mv_samp{ncr}=cell2mat(m_samp);
            samplesize_vec{ncr}=samplesize.vector;
            %mv_samp=sort(mv_samp_unsort);
            nsamp_ttl(ncr)=size(mv_samp{ncr},1);
        end
        
        % apply cpl with init vector "e"
        [chat, ~, cpl_dT, post] = ...
            cpl4c(mo.As, mo.K, e, mo.c, 'cpl', cpl_opts);
        %%% Pihat and Phat via pseudo-likelihood %%%
        [pih_PL, Phat_PL] = cplEstimParam('soft', mo.As, post, chat);
        % Error in estimation in pseudo-likelihood %
        [err_pri_PL, err_P_PL] = compParamErr2(mo.c, chat, mo.pri, pih_PL, mo.P, Phat_PL);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % Applying now our parallel algorithm %
        %%%% Sending initial estimates of the parameters to different cores %%%%%
        pi_parhat=cell(nsubsamp,1);
        plink_parhat=cell(nsubsamp,1);
        cinit_par=cell(nsubsamp,1);
        
        %%% initial class probability estimates %%%
        
        pi_init=zeros(K,1); 
        for i=1:(K-1)
            pi_init(i)=length(cinit(cinit==i))/n;
        end
        pi_init(K)=1-sum(pi_init(1:(K-1)));
        
        %%%% initial edge-probability matrix estimates %%%%
        
        plink_cc=zeros(K,K); 
        for u=1:K
            for v=1:K
                if(v>=u)
                    plink_cc(u,v)=community_sum(mo.As,u,v,cinit)/(ncomm(u,cinit)*ncomm(v,cinit));
                else
                    plink_cc(u,v)=plink_cc(v,u);
                end
            end
        end
        plink=plink_cc;
        
        %%% Assigning the initial values to each of the subsamples in each
        %%% core
        for ncr=1:nsubsamp
            plink_parhat{ncr}=plink;
            pi_parhat{ncr}=pi_init;
            cinit_par{ncr}=cinit;
        end
        Adj=mo.As;
        ngibbs=100; % number of gibbs samples for the posterior inference on z %
        nz_vec=cell(nsubsamp,1); % Each cell contains the number of zero connections in each subsample
        r=7; % global case-control rate %
        n_sampnode=lambda*r; % Size of case-control sample
        
        %%% Case-control approximation:SRS sampling over connections for each node %%%
        zero_samp=cell(nsubsamp,1);
        for ncr=1:nsubsamp
            mvsamp_vec=mv_samp{ncr};% selected nodes in each subsample
            ns_ttl=nsamp_ttl(ncr);% size of subsample in each core
            nzero_pop=zeros(ns_ttl,1);% Total number of zero connections for the selected nodes
            zerocon_samp=zeros(n_sampnode,ns_ttl);% sampled zero connections in case-control sampling
            for i=1:ns_ttl
                zero_pop=find(mo.As(mvsamp_vec(i),:)==0);
                nzero_pop(i)=length(zero_pop);
                zerocon_samp(:,i)=randsample(zero_pop,n_sampnode);
            end
            zero_samp{ncr}=zerocon_samp;
            nz_vec{ncr}=nzero_pop;
        end
        %%% Monte-Carlo EM type algorithm implemented in parallel %%%
        tic;
        est_P=cell(nsubsamp,1); % Phat in each core %
        est_pri=cell(nsubsamp,1); % Pihat in each core %
        
        %%% Iteration run in parallel over different cores %%%
        for t=1:niter
            parfor idx=1:ncore
                [est_c{idx},est_P{idx},est_pri{idx}]=parEM(ngibbs,plink_parhat{idx},cinit_par{idx},pi_parhat{idx},Adj,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
            end
            %parfor idx=11:20
                %[est_c{idx},est_P{idx},est_pri{idx}]=parEM(ngibbs,plink_parhat{idx},cinit_par{idx},pi_parhat{idx},Adj,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
            %end
            %parfor idx=21:nsubsamp
                %[est_c{idx},est_P{idx},est_pri{idx}]=parEM(ngibbs,plink_parhat{idx},cinit_par{idx},pi_parhat{idx},Adj,zero_samp{idx},mv_samp{idx},samplesize_vec{idx},nsamp_ttl(idx),n_sampnode,nz_vec{idx},K,n);
            %end
            %%% Communication Step %%%
            for idx=1:nsubsamp
                est_P{idx}=est_P{mod(idx,nsubsamp)+1};
                est_pri{idx}=est_pri{mod(idx,nsubsamp)+1};
                plink_parhat{idx}=est_P{idx};
                pi_parhat{idx}=est_pri{idx};
                cinit_par{idx}=est_c{idx};
            end
        end
        %%%%%% END of iteration %%%%%%
        
        %wtime = toc;
        %fprintf ( 1, '  Main computation:\n' );
        %fprintf ( 1, '    Wall clock time = %f seconds.\n', wtime );
        matlabpool close;
        pfinal=zeros(K,K);
        prifinal=zeros(K,1);
        
        %%% Combining estimates from different cores %%%
        for j=1:nsubsamp
            pfinal=pfinal+est_P{j};
            prifinal=prifinal+est_pri{j};
        end
        pfinal=pfinal/nsubsamp;
        prifinal=prifinal/nsubsamp;
        Fmat=cell2mat(est_c);
        cfinal=mode(Fmat,2);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%% Computing the estimation error and NMI in our algorithm %%%%
        [err_pri, err_P] = compParamErr2(mo.c, cfinal, mo.pri, prifinal, mo.P, pfinal);
        cpl_nmi(lambda_ind) = cpl_nmi(lambda_ind) + compErr(mo.c, chat);
        prierr_PL(lambda_ind)=prierr_PL(lambda_ind) + err_pri_PL;
        Perr_PL(lambda_ind)=Perr_PL(lambda_ind) + err_P_PL;
        cc_nmi(lambda_ind) = cc_nmi(lambda_ind) + compErr(mo.c, cfinal);
        prierr(lambda_ind)=prierr(lambda_ind) + err_pri;
        Perr(lambda_ind)=Perr(lambda_ind) + err_P;
    end
end
save('plinkerr.mat','Perr')
save('plinkerr_PL.mat','Perr_PL')
save('cprob_err.mat','prierr')
save('cprob_err_PL.mat','prierr_PL')
save('NMI_em.mat','cc_nmi')
save('NMI_PL.mat','cpl_nmi')
% you must include an exit command in your Matlab .m file to close
% Matlab when your program is complete.
%exit
%%%%%%%%%%%%%%%%%%%%%% END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%